package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;
import lightcontainer.utils.FileSender;

import java.io.OutputStream;

public class GetFileOkRule extends ProtocolWriter {
    // Constants
    private static final String PATTERN = "^GETFILE_OK ([^ !]{1,20}) ([0-9]{1,10})\r\n$";
    public static final String NAME = "GETFILE_OK";
    // -- params
    private static final int FILE_NAME = 0;  // Index file name hashed
    private static final int FILE_SIZE = 1;  // Index file size

    // Variables
    private String storagePath;

    // Constructors
    public GetFileOkRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    public class Result extends ProtocolWriter.ProtocolResult {
        // Variables
        private String filename;
        private int filesize;

        // Constructors
        public Result(Context context, String filename, int filesize) {
            super(context);
            this.filename = filename;
            this.filesize = filesize;
        }

        /**
         * Oh yeahh baby
         * @param writer Buffer à remplir qui sera envoyer via le réseau
         */
        @Override
        public void write(OutputStream writer) {
            super.write(writer);
            System.out.printf("Sauvegarde du fichier : %s %d\n", this.filename, this.filesize);

            FileSender fileSender = new FileSender(storagePath);
            fileSender.sendFile(getContext().getHashedFileName(this.filename), writer, this.filesize, getContext().getAesKey(), getContext().getDataString("fileIV"));
        }
    }

    @Override
    protected GetFileOkRule.Result onExecuted(Context context, String... data) {
        return new GetFileOkRule.Result(context, data[FILE_NAME], Integer.parseInt(data[FILE_SIZE]));
    }
}
