package lightcontainer.repository;

import lightcontainer.domains.Task;
import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.domains.server.MulticastServerListener;
import lightcontainer.interfaces.MulticastSPR;

import java.util.HashSet;
import java.util.Set;
// TODO : C'est genre un ClientHandlerManager quoi hein, normal qu'il fasse blinder de chose ;)
/**
 * StoreProcessorRepository
 *
 * Repository storing StorePorcessor class.
 * Contains some utility functions.
 *
 * @version     1.0
 * @since       1.0
 *
 * @see         StoreProcessor
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class StoreProcessorRepository implements AutoCloseable, MulticastSPR {
    // Variables
    private final Set<StoreProcessor> handlers;
    private MulticastServerListener server;

    // Constructor
    public StoreProcessorRepository() {
        this.handlers = new HashSet<>();
    }

    /**
     * Setter, allow to define the ServerListener of a repository.
     * & start the server.
     * @param server ServerListener to set as default.
     *
     * @since       1.0
     */
    @Override
    public void setServerListener(MulticastServerListener server) {
        this.server = server;
        new Thread(server).start();
    }

    /**
     * Add a StorePorcessor.
     * @param store Store processor to add.
     *
     * @since       1.0
     */
    @Override
    public boolean addStore(StoreProcessor store) {
        if (!this.hasDomain(store.getDomain())) {
            store.startStore();
            return this.handlers.add(store);
        }
        return false;
    }

    @Override
    public String findDomain(Task task) {
        StoreProcessor handler = findSBE(task);
        return handler == null ? null : handler.getDomain();
    }

    private StoreProcessor findSBE(Task task) {
        for (StoreProcessor handler : handlers) {
            if (handler.canProcessTask(task)) {
                return handler;
            }
        }

        return null;
    }

    @Override
    public void assignTask(String stor, Task task) {
        StoreProcessor handler = findSBE(task);
        System.out.println("Find stor : " + handler);
        handler.executeCommand(task.getContext(), task.getCommand());
    }

    @Override
    public boolean hasDomain(String domain) {
        for (StoreProcessor handler : handlers) {
            if (handler.getDomain().equals(domain)) {
                return true;
            }
        }

        return false;
    }

    @Override
    public int domainCount() {
        return handlers.size();
    }

    /**
     * AutoClosable Function
     * Closes all StoreProcessor stored in this repository and deallocates all resources.
     *
     * @since       1.0
     */
    @Override
    public void close() {
        // Stop the server.
        this.server.stop();
        // Close each client.
        this.handlers.forEach(StoreProcessor::close);
    }
}
